@@ -4,7 +4,7 @@
  * radiusd.h	Structures, prototypes and global variables
  *		for the FreeRADIUS server.
  *
- * Version:	$Id: radiusd.h,v 1.260 2008/02/11 15:19:54 aland Exp $
+ * Version:	$Id: radiusd.h,v 1.261 2008/03/07 09:49:49 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSIDH(radiusd_h, "$Id: radiusd.h,v 1.260 2008/02/11 15:19:54 aland Exp $")
+RCSIDH(radiusd_h, "$Id: radiusd.h,v 1.261 2008/03/07 09:49:49 aland Exp $")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/radpaths.h>
@@ -330,6 +330,7 @@ void			radius_signal_self(int flag);
 #define RADIUS_SIGNAL_SELF_EXIT		(1 << 2)
 #define RADIUS_SIGNAL_SELF_DETAIL	(1 << 3)
 #define RADIUS_SIGNAL_SELF_NEW_FD	(1 << 4)
+#define RADIUS_SIGNAL_SELF_MAX		(1 << 5)
 
 
 /*
@@ -348,9 +349,9 @@ int		session_zap(REQUEST *request, uint32_t nasaddr,
 			    char proto,int session_time);
 
 /* radiusd.c */
-#ifndef _LIBRADIUS
-void		debug_pair(FILE *, VALUE_PAIR *);
-#endif
+#undef debug_pair
+void		debug_pair(VALUE_PAIR *);
+void		debug_pair_list(VALUE_PAIR *);
 int		log_err (char *);
 
 /* util.c */
