@@ -1,7 +1,7 @@
 /*
  * valuepair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: valuepair.c,v 1.136 2008/01/20 09:04:14 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.138 2008/03/13 11:17:13 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: valuepair.c,v 1.136 2008/01/20 09:04:14 aland Exp $")
+RCSID("$Id: valuepair.c,v 1.138 2008/03/13 11:17:13 aland Exp $")
 
 #include	<freeradius-devel/libradius.h>
 
@@ -316,7 +316,7 @@ VALUE_PAIR *paircopy2(VALUE_PAIR *vp, int attr)
 			name_len = FR_VP_NAME_PAD;
 		}
 		
-		if ((n = malloc(sizeof(*n))) == NULL) {
+		if ((n = malloc(sizeof(*n) + name_len)) == NULL) {
 			librad_log("out of memory");
 			return first;
 		}
@@ -817,7 +817,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			cp = value;
 			length = 0;
 
-			while (*cp && (length < sizeof(vp->vp_strvalue))) {
+			while (*cp && (length < (sizeof(vp->vp_strvalue) - 1))) {
 				char c = *cp++;
 
 				if (c == '\\') {
