@@ -1,7 +1,7 @@
 /*
  * event.c	Server event handling
  *
- * Version:	$Id: event.c,v 1.83 2008/01/15 15:49:38 aland Exp $
+ * Version:	$Id: event.c,v 1.94 2008/03/15 12:04:28 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,11 +22,12 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: event.c,v 1.83 2008/01/15 15:49:38 aland Exp $")
+RCSID("$Id: event.c,v 1.94 2008/03/15 12:04:28 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
 #include <freeradius-devel/event.h>
+#include <freeradius-devel/detail.h>
 #include <freeradius-devel/radius_snmp.h>
 
 #include <freeradius-devel/rad_assert.h>
@@ -43,6 +44,7 @@ RCSID("$Id: event.c,v 1.83 2008/01/15 15:49:38 aland Exp $")
 extern pid_t radius_pid;
 extern int dont_fork;
 extern int check_config;
+extern void force_log_reopen(void);
 
 /*
  *	Ridiculous amounts of local state.
@@ -54,7 +56,6 @@ static struct timeval		now;
 static time_t			start_time;
 static int			have_children;
 static int			has_detail_listener = FALSE;
-static int			read_from_detail = TRUE;
 static int			just_started = FALSE;
 
 #ifndef __MINGW32__
@@ -87,12 +88,12 @@ static rad_listen_t	*proxy_listeners[32];
 
 static void request_post_handler(REQUEST *request);
 static void wait_a_bit(void *ctx);
-static void event_poll_fd(void *ctx);
+static void event_socket_handler(fr_event_list_t *xel, UNUSED int fd, void *ctx);
 
 static void NEVER_RETURNS _rad_panic(const char *file, unsigned int line,
 				    const char *msg)
 {
-	radlog(L_ERR, "]%s:%d] %s", file, line, msg);
+	radlog(L_ERR, "[%s:%d] %s", file, line, msg);
 	_exit(1);
 }
 
@@ -298,18 +299,9 @@ static int insert_into_proxy_hash(REQUEST *request)
 		rad_listen_t *proxy_listener;
 
 		/*
-		 *	Allocate a new proxy fd.  This function adds it
-		 *	into the list of listeners.
-		 *
-		 *	FIXME!: Call event_fd_insert()!  Otherwise this
-		 *	FD will never be used in select()!
-		 *
-		 *	We probably want to add RADIUS_SIGNAL_SELF_FD,
-		 *	which tells us to walk over the listeners,
-		 *	adding a new FD.
-		 *
-		 *	Hmm... maybe do this for the detail file, too.
-		 *	that will simplify some of the rest of the code.
+		 *	Allocate a new proxy fd.  This function adds
+		 *	it to the tail of the list of listeners.  With
+		 *	some care, this can be thread-safe.
 		 */
 		proxy_listener = proxy_new_listener();
 		if (!proxy_listener) {
@@ -324,8 +316,6 @@ static int insert_into_proxy_hash(REQUEST *request)
 		found = -1;
 		proxy = proxy_listener->fd;
 		for (i = 0; i < 32; i++) {
-			DEBUG2("PROXY %d %d", i, proxy_fds[(proxy + i) & 0x1f]);
-
 			/*
 			 *	Found a free entry.  Save the socket,
 			 *	and remember where we saved it.
@@ -342,7 +332,7 @@ static int insert_into_proxy_hash(REQUEST *request)
 		if (!fr_packet_list_socket_add(proxy_list, proxy_listener->fd)) {
 			PTHREAD_MUTEX_UNLOCK(&proxy_mutex);
 			DEBUG2("ERROR: Failed to create a new socket for proxying requests.");
-			return 0; /* leak proxy_listener */
+			return 0;
 
 		}
 
@@ -351,6 +341,12 @@ static int insert_into_proxy_hash(REQUEST *request)
 			DEBUG2("ERROR: Failed to create a new socket for proxying requests.");
 			return 0;
 		}
+
+		/*
+		 *	Signal the main thread to add the new FD to the list
+		 *	of listening FD's.
+		 */
+		radius_signal_self(RADIUS_SIGNAL_SELF_NEW_FD);
 	}
 	rad_assert(request->proxy->sockfd >= 0);
 
@@ -876,11 +872,23 @@ static void wait_a_bit(void *ctx)
 		 *	Request still has more time.  Continue
 		 *	waiting.
 		 */
-		if (timercmp(&now, &when, <)) {
+		if (timercmp(&now, &when, <) ||
+		    ((request->listener->type == RAD_LISTEN_DETAIL) &&
+		     (request->child_state == REQUEST_QUEUED))) {
 			if (request->delay < (USEC / 10)) {
 				request->delay = USEC / 10;
 			}
 			request->delay += request->delay >> 1;
+
+			/*
+			 *	Cap wait at some sane value for detail
+			 *	files.
+			 */
+			if ((request->listener->type == RAD_LISTEN_DETAIL) &&
+			    (request->delay > (request->root->max_request_time * USEC))) {
+				request->delay = request->root->max_request_time * USEC;
+			}
+
 			request->when = now;
 			tv_add(&request->when, request->delay);
 			callback = wait_a_bit;
@@ -1160,7 +1168,7 @@ static int successfully_proxied_request(REQUEST *request)
 	 *
 	 *	FIXME: This should really be a serious error.
 	 */
-	if (request->in_proxy_hash || request->proxy) {
+	if (request->in_proxy_hash) {
 		return 0;
 	}
 
@@ -1418,7 +1426,7 @@ static void request_post_handler(REQUEST *request)
 	}
 
 	if (request->root->proxy_requests &&
-	    !request->proxy &&
+	    !request->in_proxy_hash &&
 	    (request->reply->code == 0) &&
 	    (request->packet->dst_port != 0) &&
 	    (request->packet->code != PW_STATUS_SERVER)) {
@@ -1467,7 +1475,8 @@ static void request_post_handler(REQUEST *request)
 	/*
 	 *	Access-Requests get delayed or cached.
 	 */
-	if (request->packet->code == PW_AUTHENTICATION_REQUEST) {
+	switch (request->packet->code) {
+	case PW_AUTHENTICATION_REQUEST:
 		gettimeofday(&request->next_when, NULL);
 
 		if (request->reply->code == 0) {
@@ -1522,21 +1531,25 @@ static void request_post_handler(REQUEST *request)
 		request->next_when.tv_sec += request->root->cleanup_delay;
 		request->next_callback = cleanup_delay;
 		child_state = REQUEST_CLEANUP_DELAY;
+		break;
 
-	} else if (request->packet->code == PW_ACCOUNTING_REQUEST) {
+	case PW_ACCOUNTING_REQUEST:
 		request->next_callback = NULL; /* just to be safe */
 		child_state = REQUEST_DONE;
+		break;
 
 		/*
 		 *	FIXME: Status-Server should probably not be
 		 *	handled here...
 		 */
-	} else if (request->packet->code == PW_STATUS_SERVER) {
+	case PW_STATUS_SERVER:
 		request->next_callback = NULL;
 		child_state = REQUEST_DONE;
+		break;
 
-	} else {
+	default:
 		rad_panic("Unknown packet type");
+		break;
 	}
 
 	/*
@@ -1584,6 +1597,11 @@ static void request_post_handler(REQUEST *request)
 	DEBUG2("Finished request %d.", request->number);
 
 	request->child_state = child_state;
+
+	/*
+	 *	Single threaded mode: update timers now.
+	 */
+	if (!have_children) wait_a_bit(request);
 }
 
 
@@ -1866,7 +1884,8 @@ int received_request(rad_listen_t *listener,
 	/*
 	 *	We may want to quench the new request.
 	 */
-	if (!can_handle_new_request(packet, client, root)) {
+	if ((listener->type != RAD_LISTEN_DETAIL) &&
+	    !can_handle_new_request(packet, client, root)) {
 		return 0;
 	}
 
@@ -2112,12 +2131,21 @@ REQUEST *received_proxy_response(RADIUS_PACKET *packet)
 }
 
 
+static void event_detail_timer(void *ctx)
+{
+	rad_listen_t *listener = ctx;
+	RAD_REQUEST_FUNP fun;
+	REQUEST *request;
+
+	if (listener->recv(listener, &fun, &request)) {
+		if (!thread_pool_addrequest(request, fun)) {
+			request->child_state = REQUEST_DONE;
+		}
+	}
+}
+
 static void handle_signal_self(int flag)
 {
-	if ((flag & RADIUS_SIGNAL_SELF_DETAIL) != 0) {
-		read_from_detail = TRUE;
-	}
-
 	if ((flag & (RADIUS_SIGNAL_SELF_EXIT | RADIUS_SIGNAL_SELF_TERM)) != 0) {
 		if ((flag & RADIUS_SIGNAL_SELF_EXIT) != 0) {
 			fr_event_loop_exit(el, 1);
@@ -2146,6 +2174,53 @@ static void handle_signal_self(int flag)
 
 		fr_event_loop_exit(el, 0x80);
 	}
+
+	if ((flag & RADIUS_SIGNAL_SELF_DETAIL) != 0) {
+		rad_listen_t *this;
+		
+		for (this = mainconfig.listen;
+		     this != NULL;
+		     this = this->next) {
+			int delay;
+			struct timeval when;
+
+			if (this->type != RAD_LISTEN_DETAIL) continue;
+			
+			delay = detail_delay(this);
+			if (!delay) continue;
+
+			fr_event_now(el, &now);
+			when = now;
+			tv_add(&when, delay);
+
+			if (delay > 100000) {
+				DEBUG2("Delaying next detail event for %d.%01u seconds.",
+				       delay / USEC, (delay % USEC) / 100000);
+			}
+
+			if (!fr_event_insert(el, event_detail_timer, this,
+					     &when, NULL)) {
+				radlog(L_ERR, "Failed remembering timer");
+				exit(1);
+			}
+		}
+	}
+
+	if ((flag & RADIUS_SIGNAL_SELF_NEW_FD) != 0) {
+		rad_listen_t *this;
+		
+		for (this = mainconfig.listen;
+		     this != NULL;
+		     this = this->next) {
+			if (this->type != RAD_LISTEN_PROXY) continue;
+			
+			if (!fr_event_fd_insert(el, 0, this->fd,
+						event_socket_handler, this)) {
+				radlog(L_ERR, "Failed remembering handle for proxy socket!");
+				exit(1);
+			}
+		}
+	}
 }
 
 #ifdef __MINGW32__
@@ -2162,17 +2237,6 @@ void radius_signal_self(int flag)
 	ssize_t rcode;
 	uint8_t buffer[16];
 
-	/*
-	 *	The caller is telling us it's OK to read from the
-	 *	detail files.  However, we're not even listening on
-	 *	the detail files.  Therefore, suppress the flag to
-	 *	avoid bothering the main worker thread.
-	 */
-	if ((flag == RADIUS_SIGNAL_SELF_DETAIL) &&
-	    !has_detail_listener) {
-		return;
-	}
-
 	/*
 	 *	The read MUST be non-blocking for this to work.
 	 */
@@ -2180,15 +2244,13 @@ void radius_signal_self(int flag)
 	if (rcode > 0) {
 		ssize_t i;
 
-		for (i = 1; i < rcode; i++) {
+		for (i = 0; i < rcode; i++) {
 			buffer[0] |= buffer[i];
 		}
+	} else {
+		buffer[0] = 0;
 	}
 
-#ifndef NDEBUG
-	memset(buffer + 1, 0, sizeof(buffer) - 1);
-#endif
-
 	buffer[0] |= flag;
 
 	write(self_pipe[1], buffer, 1);
@@ -2199,7 +2261,7 @@ static void event_signal_handler(UNUSED fr_event_list_t *xel,
 				 UNUSED int fd, UNUSED void *ctx)
 {
 	ssize_t i, rcode;
-	char buffer[32];
+	uint8_t buffer[32];
 
 	rcode = read(self_pipe[0], buffer, sizeof(buffer));
 	if (rcode <= 0) return;
@@ -2207,7 +2269,7 @@ static void event_signal_handler(UNUSED fr_event_list_t *xel,
 	/*
 	 *	Merge pending signals.
 	 */
-	for (i = 1; i < rcode; i++) {
+	for (i = 0; i < rcode; i++) {
 		buffer[0] |= buffer[i];
 	}
 
@@ -2215,99 +2277,9 @@ static void event_signal_handler(UNUSED fr_event_list_t *xel,
 }
 #endif
 
-/*
- *	The given FD is closed.  Delete it from the list of FD's to poll,
- *	and set a timer to periodically look for an FD again.
- */
-static void event_reset_fd(rad_listen_t *listener, int fd, int usec)
-{
-	struct timeval when;
-	gettimeofday(&when, NULL);
 
-	tv_add(&when, usec);	
-
-	if (!fr_event_fd_delete(el, 0, fd)) {
-		rad_panic("Failed deleting fd");
-	}
-	
-	if (!fr_event_insert(el, event_poll_fd, listener,
-			       &when, NULL)) {
-		rad_panic("Failed inserting event");
-	}
-}
-
-
-/*
- *	Reads from the detail file if possible, OR sets up a timeout
- *	to see if the detail file is ready for reading.
- *
- *	FIXME: On Linux, use inotify to watch the detail file.  This
- *	
- */
-static void event_detail_handler(fr_event_list_t *xel, int fd, void *ctx)
-{
-	rad_listen_t *listener = ctx;
-	RAD_REQUEST_FUNP fun;
-	REQUEST *request;
-
-	rad_assert(xel == el);
-	rad_assert(fd == listener->fd);
-
-	xel = xel;
-
-	/*
-	 *	We're too busy to read from the detail file.  Stop
-	 *	watching the file, and instead go back to polling.
-	 */
-	if (!read_from_detail) {
-		event_reset_fd(listener, fd, USEC);
-		return;
-	}
-	
-	/*
-	 *	FIXME: Put this somewhere else, where it isn't called
-	 *	all of the time...
-	 */
-#if !defined(HAVE_PTHREAD_H) && defined(WNOHANG)
-	/*
-	 *	If there are no child threads, then there may
-	 *	be child processes.  In that case, wait for
-	 *	their exit status, and throw that exit status
-	 *	away.  This helps get rid of zxombie children.
-	 */
-	while (waitpid(-1, &argval, WNOHANG) > 0) {
-		/* do nothing */
-	}
-#endif
-
-	/*
-	 *	Didn't read anything from the detail file.
-	 *	Wake up in one second to check it again.
-	 */
-	if (!listener->recv(listener, &fun, &request)) {
-		event_reset_fd(listener, fd, USEC);
-		return;
-	}
-
-	/*
-	 *	Drop the request into the thread pool,
-	 *	and let the thread pool take care of
-	 *	doing something with it.
-	 */
-	if (!thread_pool_addrequest(request, fun)) {
-		request->child_state = REQUEST_DONE;
-	}
-
-	/*
-	 *	Reset based on the fd we were given, as detail_recv()
-	 *	may have read all of the file, and closed it.  It then
-	 *	sets listener->fd = -1.
-	 */
-	event_reset_fd(listener, fd, *(int *) listener->data);
-}
-
-
-static void event_socket_handler(fr_event_list_t *xel, UNUSED int fd, void *ctx)
+static void event_socket_handler(fr_event_list_t *xel, UNUSED int fd,
+				 void *ctx)
 {
 	rad_listen_t *listener = ctx;
 	RAD_REQUEST_FUNP fun;
@@ -2340,92 +2312,77 @@ static void event_socket_handler(fr_event_list_t *xel, UNUSED int fd, void *ctx)
 	if (!thread_pool_addrequest(request, fun)) {
 		request->child_state = REQUEST_DONE;
 	}
-
-	/*
-	 *	If we've read a packet from a socket OTHER than the
-	 *	detail file, we start ignoring the detail file.
-	 *
-	 *	When the child thread pulls the request off of the
-	 *	queues, it will check if the queues are empty.  Once
-	 *	all queues are empty, it will signal us to read the
-	 *	detail file again.
-	 */
-	read_from_detail = FALSE;
 }
 
 
 /*
- *	This function is called periodically to see if a file
- *	descriptor is available for reading.
+ *	This function is called periodically to see if any FD's are
+ *	available for reading.
  */
-static void event_poll_fd(void *ctx)
+static void event_poll_fds(UNUSED void *ctx)
 {
 	int rcode;
 	RAD_REQUEST_FUNP fun;
 	REQUEST *request;
-	rad_listen_t *listener = ctx;
-	fr_event_fd_handler_t handler;
+	rad_listen_t *this;
+	struct timeval when;
 
-	/*
-	 *	Ignore the detail file if we're busy with other work.
-	 */
-	if ((listener->type == RAD_LISTEN_DETAIL) && !read_from_detail) {
-		goto reset;
-	}
+	fr_event_now(el, &now);
+	when = now;
+	when.tv_sec += 1;
 
-	/*
-	 *	Try to read a RADIUS packet.  This also opens the FD.
-	 *
-	 *	FIXME: This does poll AND receive.
-	 */
-	rcode = listener->recv(listener, &fun, &request);
+	for (this = mainconfig.listen; this != NULL; this = this->next) {
+		if (this->fd >= 0) continue;
 
-	/*
-	 *	We read a request.  Add it to the list for processing.
-	 */
-	if (rcode == 1) {
+		/*
+		 *	Try to read something.
+		 *
+		 *	FIXME: This does poll AND receive.
+		 */
+		rcode = this->recv(this, &fun, &request);
+		if (!rcode) continue;
+		
 		rad_assert(fun != NULL);
 		rad_assert(request != NULL);
-
+			
 		if (!thread_pool_addrequest(request, fun)) {
 			request->child_state = REQUEST_DONE;
 		}
-	}
 
-	/*
-	 *	Still no FD, OR the fd was read completely, and then
-	 *	closed.  Re-set the polling timer and return.
-	 */
-	if (listener->fd < 0) {
-		struct timeval when;
+		/*
+		 *	We have an FD.  Start watching it.
+		 */
+		if (this->fd >= 0) {
+			/*
+			 *	... unless it's a detail file.  In
+			 *	that case, we rely on the signal to
+			 *	self to know when to continue
+			 *	processing the detail file.
+			 */
+			if (this->type == RAD_LISTEN_DETAIL) continue;
 
-	reset:
-		gettimeofday(&when, NULL);
-		when.tv_sec += 1;
-		
-		if (!fr_event_insert(el, event_poll_fd, listener,
-				       &when, NULL)) {
-			rad_panic("Failed inserting event");
+			/*
+			 *	FIXME: this should be SNMP handler,
+			 *	and we should do SOMETHING when the
+			 *	fd is closed!
+			 */
+			if (!fr_event_fd_insert(el, 0, this->fd,
+						event_socket_handler, this)) {
+				char buffer[256];
+				
+				this->print(this, buffer, sizeof(buffer));
+				rad_panic("Failed creating handler for snmp");
+			}
 		}
-
-		return;
-	}
-
-	if (listener->type == RAD_LISTEN_DETAIL) {
-		handler = event_detail_handler;
-	} else {
-		handler = event_socket_handler;
 	}
 
 	/*
-	 *	We have an FD.  Start watching it.
+	 *	Reset the poll.
 	 */
-	if (!fr_event_fd_insert(el, 0, listener->fd,
-				  handler, listener)) {
-		char buffer[256];
-		
-		listener->print(listener, buffer, sizeof(buffer));
-		rad_panic("Failed creating handler for socket");
+	if (!fr_event_insert(el, event_poll_fds, NULL,
+			     &when, NULL)) {
+		radlog(L_ERR, "Failed creating handler");
+		exit(1);
 	}
 }
 
@@ -2442,9 +2399,10 @@ static void event_status(struct timeval *wake)
 
 	if (!wake) {
 		DEBUG("Ready to process requests.");
+
 	} else if ((wake->tv_sec != 0) ||
 		   (wake->tv_usec >= 100000)) {
-		DEBUG("Waking up in %d.%01u seconds.\n",
+		DEBUG("Waking up in %d.%01u seconds.",
 		      (int) wake->tv_sec, (unsigned int) wake->tv_usec / 100000);
 	}
 }
@@ -2456,6 +2414,7 @@ static void event_status(struct timeval *wake)
 int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 {
 	int i;
+	int has_snmp_listener = FALSE;
 	rad_listen_t *this, *head = NULL;
 
 	if (el) return 0;
@@ -2494,6 +2453,12 @@ int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 #endif
 	}
 
+	/*
+	 *	Just before we spawn the child threads, force the log
+	 *	subsystem to re-open the log file for every write.
+	 */
+	if (spawn_flag) force_log_reopen();
+
 #ifdef HAVE_PTHREAD_H
 	if (thread_pool_init(cs, spawn_flag) < 0) {
 		exit(1);
@@ -2548,11 +2513,6 @@ int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 	
 	/*
 	 *	Add all of the sockets to the event loop.
-	 *
-	 *	FIXME: New proxy sockets aren't currently added to the
-	 *	event loop.  They're allocated in a child thread, and
-	 *	we don't have (or want) a mutex around the event
-	 *	handling code.
 	 */
 	for (this = head;
 	     this != NULL;
@@ -2567,6 +2527,11 @@ int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 			has_detail_listener = TRUE;
 			break;
 
+		case RAD_LISTEN_SNMP:
+			DEBUG("Listening on SNMP %s", buffer);
+			has_snmp_listener = TRUE;
+			break;
+
 		case RAD_LISTEN_PROXY:
 			rad_assert(proxy_fds[this->fd & 0x1f] == -1);
 			rad_assert(proxy_listeners[this->fd & 0x1f] == NULL);
@@ -2577,11 +2542,7 @@ int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 							 this->fd)) {
 				rad_assert(0 == 1);
 			}
-			break;
-
-		case RAD_LISTEN_SNMP:
-			DEBUG("Listening on SNMP %s", buffer);
-			break;
+			/* FALL-THROUGH */
 
 		default:
 			DEBUG("Listening on %s", buffer);
@@ -2594,16 +2555,6 @@ int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 		 *	and detail file fd's.
 		 */
 		if (this->fd < 0) {
-			struct timeval when;
-
-			gettimeofday(&when, NULL);
-			when.tv_sec += 1;
-
-			if (!fr_event_insert(el, event_poll_fd, this,
-					       &when, NULL)) {
-				radlog(L_ERR, "Failed creating handler");
-				exit(1);
-			}
 			continue;
 		}
 
@@ -2623,6 +2574,19 @@ int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 		}
 	}
 
+	if (has_detail_listener || has_snmp_listener) {
+		struct timeval when;
+		
+		gettimeofday(&when, NULL);
+		when.tv_sec += 1;
+		
+		if (!fr_event_insert(el, event_poll_fds, NULL,
+				     &when, NULL)) {
+			radlog(L_ERR, "Failed creating handler");
+			exit(1);
+		}
+	}
+
 	mainconfig.listen = head;
 
 	return 1;
