@@ -1,7 +1,7 @@
 /*
  * realms.c	Realm handling code
  *
- * Version:     $Id: realms.c,v 1.43 2008/01/09 13:39:13 aland Exp $
+ * Version:     $Id: realms.c,v 1.44 2008/02/10 15:23:33 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: realms.c,v 1.43 2008/01/09 13:39:13 aland Exp $")
+RCSID("$Id: realms.c,v 1.44 2008/02/10 15:23:33 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/rad_assert.h>
@@ -1323,10 +1323,18 @@ int realms_init(CONF_SECTION *config)
 REALM *realm_find(const char *name)
 {
 	REALM myrealm;
-
+	REALM *realm;
+	
 	if (!name) name = "NULL";
 
 	myrealm.name = name;
+	realm = rbtree_finddata(realms_byname, &myrealm);
+	if (realm) return realm;
+
+	/*
+	 *	Couldn't find a realm.  Look for DEFAULT.
+	 */
+	myrealm.name = "DEFAULT";
 	return rbtree_finddata(realms_byname, &myrealm);
 }
 
