@@ -1,7 +1,7 @@
 /*
  * mem.c  Memory allocation, deallocation stuff.
  *
- * Version:     $Id: mem.c,v 1.26 2007/12/20 15:33:03 aland Exp $
+ * Version:     $Id: mem.c,v 1.28 2008/02/10 09:18:10 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: mem.c,v 1.26 2007/12/20 15:33:03 aland Exp $")
+RCSID("$Id: mem.c,v 1.28 2008/02/10 09:18:10 aland Exp $")
 
 #include <stdio.h>
 #include "rlm_eap.h"
@@ -176,7 +176,7 @@ static uint32_t eap_rand(fr_randctx *ctx)
 	uint32_t num;
 
 	num = ctx->randrsl[ctx->randcnt++];
-	if (ctx->randcnt == 256) {
+	if (ctx->randcnt >= 256) {
 		ctx->randcnt = 0;
 		fr_isaac(ctx);
 	}
@@ -264,17 +264,19 @@ int eaplist_add(rlm_eap_t *inst, EAP_HANDLER *handler)
 	 *	It will be modified slightly per round trip, but less so
 	 *	than in 1.x.
 	 */
-	if (memcmp(handler->state, "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000", 16) == 0) {
+	if (handler->trips == 0) {
 		int i;
 
 		for (i = 0; i < 4; i++) {
 			uint32_t lvalue;
-			
+
 			lvalue = eap_rand(&inst->rand_pool);
+
 			memcpy(handler->state + i * 4, &lvalue,
 			       sizeof(lvalue));
-		}
+		}		
 	}
+
 	memcpy(state->vp_octets, handler->state, sizeof(handler->state));
 	state->length = EAP_STATE_LEN;
 
@@ -317,8 +319,10 @@ int eaplist_add(rlm_eap_t *inst, EAP_HANDLER *handler)
 	pthread_mutex_unlock(&(inst->session_mutex));
 
 	if (!status) {
-		radlog(L_ERR, "rlm_eap: Failed to remember handler!");
-		eap_handler_free(handler);
+		char buffer[1024];
+
+		vp_prints_value(buffer, sizeof(buffer), state, 0);
+		radlog(L_ERR, "rlm_eap: Failed to remember handler with State %s!", buffer);
 		return 0;
 	}
 
